;; emacs local configuration settings for libpsl source
;; surmised by dkg on 2014-03-21 14:35:49-0400

((c-mode
  (indent-tabs-mode . t)
  (tab-width . 4)
  (c-basic-offset . 4)
  (c-file-style . "linux"))
 )
